// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16SDI/16SDI_Linux_4.x.x.x_GSC_DN/driver/device.c $
// $Rev: 51954 $
// $Date: 2022-10-25 08:44:06 -0500 (Tue, 25 Oct 2022) $

// 16SDI: Device Driver: source file

#include "main.h"



// variables ******************************************************************

const gsc_dev_id_t	dev_id_list[]	=
{
	// model		Vendor	Device	SubVen	SubDev	type
	{ "6SDI",		0x10B5, 0x9080, 0x10B5, 0x2408, GSC_DEV_TYPE_6SDI		},
	{ "16SDI",		0x10B5, 0x9080, 0x10B5, 0x2371, GSC_DEV_TYPE_16SDI		},

	// There is no way to distinguish PMC-16SDI-HS from PMC-16HSDI.

	{ "16SDI-HS",	0x10B5, 0x9080, 0x10B5, 0x2449, GSC_DEV_TYPE_16SDI_HS	},	// PCI: BRR.D8=0
	//				0x10B5, 0x9080, 0x10B5, 0x2306								// PMC
	{ "16HSDI",		0x10B5, 0x9080, 0x10B5, 0x2306, GSC_DEV_TYPE_16HSDI		},	// PMC
	//				0x10B5, 0x9080, 0x10B5, 0x2449								// PCI: BRR.D8=1
	{ NULL }
};



//*****************************************************************************
static void _auto_calibrate_compute(dev_data_t* dev)
{
	switch (dev->board_type)
	{
		default:

			dev->cache.auto_cal_ms	= 10 * 1000;	// worst case supported.
			break;

		case GSC_DEV_TYPE_6SDI:

			dev->cache.auto_cal_ms	= 5 * 1000;
			break;

		case GSC_DEV_TYPE_16SDI:

			if (dev->cache.fifo_size > _64K)
				dev->cache.auto_cal_ms	= 10 * 1000;
			else
				dev->cache.auto_cal_ms	= 5 * 1000;

			break;

		case GSC_DEV_TYPE_16SDI_HS:
		case GSC_DEV_TYPE_16HSDI:

			if (dev->cache.fifo_size > _64K)
				dev->cache.auto_cal_ms	= 4 * 1000;
			else
				dev->cache.auto_cal_ms	= 5 * 1000;

			break;
	}
}



//*****************************************************************************
static void _device_type_compute(dev_data_t* dev)
{
	u16	sdid;

	sdid	= os_reg_pci_rx_u16(dev, 1, 0x2E);

	switch (sdid)
	{
		default:
		case 0x2306:	// There is no way to distinguish PMC-16SDI-HS from PMC-16HSDI.
		case 0x2371:	// PC104P-16SDI or PCI-16SDI
		case 0x2408:	// PMC-6SDI

			// These have already been decided.
			break;

		case 0x2449:

			if (dev->cache.gsc_brr_32 & D8)
			{
				dev->board_type	= GSC_DEV_TYPE_16HSDI;		// PCI-16HSDI
				dev->model		= "16HSDI";
			}
			else
			{
				dev->board_type	= GSC_DEV_TYPE_16SDI_HS;	// PCI-16SDI-HS
				dev->model		= "16SDI-HS";
			}

			break;
	}
}



//*****************************************************************************
static void _channels_compute(dev_data_t* dev)
{
	u32	brr;
	int	d16		= 0;
	int	d17		= 0;
	int	max		= 0;

	brr	= os_reg_mem_rx_u32(dev, dev->vaddr.gsc_brr_32);

	if (dev->cache.fifo_size == _64K)
	{
		max		= 6;
		d16		= 4;
		d17		= 2;
	}
	else
	{
		switch (dev->board_type)
		{
			case GSC_DEV_TYPE_16SDI:

				max	= 16;
				d16	= 8;
				d17	= 4;
				break;

			default:
			case GSC_DEV_TYPE_16HSDI:
			case GSC_DEV_TYPE_16SDI_HS:

				max	= 8;
				d16	= 4;
				d17	= 0;
				break;
		}
	}

	if (dev->cache.fifo_size <= _64K)
		dev->cache.channel_groups	= 2;
	else
		dev->cache.channel_groups	= 4;

	if ((d17) && (brr & 0x20000))
		dev->cache.channel_qty	= d17;
	else if ((d16) && (brr & 0x10000))
		dev->cache.channel_qty	= d16;
	else
		dev->cache.channel_qty	= max;

	dev->cache.channels_max	= max;
	dev->cache.brr_chan_d16	= d16;
	dev->cache.brr_chan_d17	= d17;
}



//*****************************************************************************
static void _fifo_size_compute(dev_data_t* dev)
{
	u32	btr;
	u32	lvl;
	u16	sdid;

	sdid	= os_reg_pci_rx_u16(dev, 1, 0x2E);

	if (sdid == 0x2449)			// PCI-16SDI-HS or PCI-16HSDI
	{
		dev->cache.fifo_size		= _256K;
		dev->cache.threshold_mask	= 0x3FFFF;
	}
	else if (sdid == 0x2306)	// PMC-16SDI-HS or PMC-16HSDI
	{
		dev->cache.fifo_size		= _64K;
		dev->cache.threshold_mask	= 0xFFFF;
	}
	else if (sdid == 0x2408)	// PMC-6SDI
	{
		dev->cache.fifo_size		= _64K;
		dev->cache.threshold_mask	= 0xFFFF;
	}
	else						// 16SDI
	{
		btr	= os_reg_mem_rx_u32(dev, dev->vaddr.gsc_btr_32);
		lvl	= btr & 0x3FFFF;

		if (lvl == 0x3FFFE)
		{
			dev->cache.fifo_size		= _256K;
			dev->cache.threshold_mask	= 0x3FFFF;
		}
		else if (lvl == 0xFFFE)
		{
			dev->cache.fifo_size		= _64K;
			dev->cache.threshold_mask	= 0xFFFF;
		}
		else if (lvl > 0xFFFF)
		{
			dev->cache.fifo_size		= _256K;
			dev->cache.threshold_mask	= 0x3FFFF;
		}
		else
		{
			dev->cache.fifo_size		= _64K;
			dev->cache.threshold_mask	= 0xFFFF;
		}
	}
}



//*****************************************************************************
static void _form_factor_compute(dev_data_t* dev)
{
	u16	sdid;

	sdid	= os_reg_pci_rx_u16(dev, 1, 0x2E);

	switch (sdid)
	{
		default:

			dev->cache.form_factor	= SDI_FORM_FACTOR_UNKNOWN;
			break;

		case 0x2306:	// PMC-16SDI-HS or PMC-16HSDI

			dev->cache.form_factor	= SDI_FORM_FACTOR_PMC;
			break;

		case 0x2371:	// PC104P-16SDI or PCI-16SDI

			if (dev->cache.fifo_size == _256K)
				dev->cache.form_factor	= SDI_FORM_FACTOR_PCI;		// PCI-16SDI
			else
				dev->cache.form_factor	= SDI_FORM_FACTOR_PC104P;	// PC104P-16SDI

			break;

		case 0x2408:	// PMC-6SDI

			dev->cache.form_factor	= SDI_FORM_FACTOR_PMC;
			break;

		case 0x2449:	// PCI-16SDI-HS or PCI-16HSDI

			dev->cache.form_factor	= SDI_FORM_FACTOR_PCI;
			break;
	}
}



//*****************************************************************************
static void _irq_mask_compute(dev_data_t* dev)
{
	u16	sdid;

	sdid	= os_reg_pci_rx_u16(dev, 1, 0x2E);

	switch (sdid)
	{
		default:
		case 0x2306:	// PMC-16SDI-HS or PMC-16HSDI
		case 0x2371:	// PC104P-16SDI or PCI-16SDI
		case 0x2408:	// PMC-6SDI

			dev->cache.irq_mask	= 0x7F;
			break;

		case 0x2449:	// PCI-16SDI-HS or PCI-16HSDI

			if (dev->board_type == GSC_DEV_TYPE_16HSDI)
				dev->cache.irq_mask	= 0x1F;
			else
				dev->cache.irq_mask	= 0x7F;

			break;
	}
}



//*****************************************************************************
static void _rate_generators_compute(dev_data_t* dev)
{
	dev->cache.rate_gen_nrate_min	= 0;
	dev->cache.rate_gen_nrate_max	= 511;
	dev->cache.rate_gen_nrate_mask	= 0x1FF;

	switch (dev->board_type)
	{
		default:
		case GSC_DEV_TYPE_6SDI:

			dev->cache.rate_gen_fgen_max	= _16MHZ;
			dev->cache.rate_gen_fgen_min	= _8MHZ;
			dev->cache.rate_gen_qty			= 2;
			break;

		case GSC_DEV_TYPE_16SDI:

			if (dev->cache.fifo_size > _64K)
				dev->cache.rate_gen_qty	= 4;
			else
				dev->cache.rate_gen_qty	= 2;

			dev->cache.rate_gen_fgen_max	= _16MHZ;
			dev->cache.rate_gen_fgen_min	= _8MHZ;
			break;

		case GSC_DEV_TYPE_16SDI_HS:
		case GSC_DEV_TYPE_16HSDI:

			if (dev->cache.fifo_size > _64K)
				dev->cache.rate_gen_qty		= 4;
			else
				dev->cache.rate_gen_qty		= 2;

			dev->cache.rate_gen_fgen_max	= _38_4MHZ;
			dev->cache.rate_gen_fgen_min	= _19_2MHZ;
			break;
	}
}



//*****************************************************************************
static void _rate_divisors_compute(dev_data_t* dev)
{
	switch (dev->board_type)
	{
		default:
		case GSC_DEV_TYPE_6SDI:
		case GSC_DEV_TYPE_16SDI:

			dev->cache.rate_div_ndiv_min	= 1;
			dev->cache.rate_div_ndiv_max	= 32;
			dev->cache.rate_div_ndiv_mask	= 0x3F;
			break;

		case GSC_DEV_TYPE_16SDI_HS:
		case GSC_DEV_TYPE_16HSDI:

			dev->cache.rate_div_ndiv_min	= 0;
			dev->cache.rate_div_ndiv_max	= 20;
			dev->cache.rate_div_ndiv_mask	= 0x1F;
			break;
	}
}



//*****************************************************************************
static void _sample_rates_compute(dev_data_t* dev)
{
	switch (dev->board_type)
	{
		default:
		case GSC_DEV_TYPE_6SDI:	// No 256K FIFO version of this device.
		case GSC_DEV_TYPE_16SDI:

			dev->cache.fsamp_max	= 220000L;
			dev->cache.fsamp_min	=   5000L;
			break;

		case GSC_DEV_TYPE_16SDI_HS:
		case GSC_DEV_TYPE_16HSDI:

			dev->cache.fsamp_max	= 1100000L;
			dev->cache.fsamp_min	=   30000L;
			break;
	}
}



/******************************************************************************
*
*	Function:	dev_device_create
*
*	Purpose:
*
*		Do everything needed to setup and use the given device.
*
*	Arguments:
*
*		dev		The structure to initialize.
*
*	Returned:
*
*		0		All is well.
*		< 0		An appropriate error status.
*
******************************************************************************/

int dev_device_create(dev_data_t* dev)
{
	static const gsc_bar_maps_t	bar_map	=
	{
		{
			// mem	io	rw
			{ 1,	0,	GSC_REG_TYPE_ACCESS_RO },	// BAR 0: PLX registers, memory mapped
			{ 0,	0,	GSC_REG_TYPE_ACCESS_RO },	// BAR 1: PLX registers, I/O mapped
			{ 1,	0,	GSC_REG_TYPE_ACCESS_RW },	// BAR 2: GSC registers, memory mapped
			{ 0,	0,	GSC_REG_TYPE_ACCESS_RO },	// BAR 3: unused
			{ 0,	0,	GSC_REG_TYPE_ACCESS_RO },	// BAR 4: unused
			{ 0,	0,	GSC_REG_TYPE_ACCESS_RO }	// BAR 5: unused
		}
	};

	u32	dma;
	int	ret;

	for (;;)	// A convenience loop.
	{
		// Verify some macro contents.
		ret	= gsc_macro_test_base_name(SDI_BASE_NAME);
		if (ret)	break;

		ret	= gsc_macro_test_model();
		if (ret)	break;

		// PCI setup.
		ret	= os_pci_dev_enable(&dev->pci);
		if (ret)	break;

		ret	= os_pci_master_set(&dev->pci);
		if (ret)	break;

		// Control ISR access to the device and data structure.
		ret	= os_spinlock_create(&dev->spinlock);
		if (ret)	break;

		// Control access to the device and data structure.
		ret	= os_sem_create(&dev->sem);
		if (ret)	break;

		// Access the BAR regions.
		ret	= gsc_bar_create(dev, &dev->bar, &bar_map);
		if (ret)	break;

		// Firmware access.
		dev->vaddr.gsc_bcr_32		= GSC_VADDR(dev, GSC_REG_OFFSET(SDI_GSC_BCR));
		dev->vaddr.gsc_brr_32		= GSC_VADDR(dev, GSC_REG_OFFSET(SDI_GSC_BRR));
		dev->vaddr.gsc_bsr_32		= GSC_VADDR(dev, GSC_REG_OFFSET(SDI_GSC_BSR));
		dev->vaddr.gsc_btr_32		= GSC_VADDR(dev, GSC_REG_OFFSET(SDI_GSC_BTR));
		dev->vaddr.gsc_idbr_32		= GSC_VADDR(dev, GSC_REG_OFFSET(SDI_GSC_IDBR));
		dev->vaddr.gsc_rar_32		= GSC_VADDR(dev, GSC_REG_OFFSET(SDI_GSC_RAR));
		dev->vaddr.gsc_rcar_32		= GSC_VADDR(dev, GSC_REG_OFFSET(SDI_GSC_RCAR));
		dev->vaddr.gsc_rcbr_32		= GSC_VADDR(dev, GSC_REG_OFFSET(SDI_GSC_RCBR));
		dev->vaddr.gsc_rccr_32		= GSC_VADDR(dev, GSC_REG_OFFSET(SDI_GSC_RCCR));
		dev->vaddr.gsc_rcdr_32		= GSC_VADDR(dev, GSC_REG_OFFSET(SDI_GSC_RCDR));
		dev->vaddr.gsc_rd01r_32		= GSC_VADDR(dev, GSC_REG_OFFSET(SDI_GSC_RD01R));
		dev->vaddr.gsc_rd23r_32		= GSC_VADDR(dev, GSC_REG_OFFSET(SDI_GSC_RD23R));
		dev->vaddr.gsc_rd45r_32		= GSC_VADDR(dev, GSC_REG_OFFSET(SDI_GSC_RD45R));
		dev->vaddr.gsc_rd67r_32		= GSC_VADDR(dev, GSC_REG_OFFSET(SDI_GSC_RD67R));
		dev->vaddr.gsc_rd89r_32		= GSC_VADDR(dev, GSC_REG_OFFSET(SDI_GSC_RD89R));
		dev->vaddr.gsc_rdabr_32		= GSC_VADDR(dev, GSC_REG_OFFSET(SDI_GSC_RDABR));
		dev->vaddr.gsc_rdcdr_32		= GSC_VADDR(dev, GSC_REG_OFFSET(SDI_GSC_RDCDR));
		dev->vaddr.gsc_rdefr_32		= GSC_VADDR(dev, GSC_REG_OFFSET(SDI_GSC_RDEFR));

		// Data cache initialization.
		dev->cache.gsc_brr_32		= os_reg_mem_rx_u32(dev, dev->vaddr.gsc_brr_32);
		dev->cache.dmdma			= (dev->cache.gsc_brr_32 & 0x8000) ? 1 : 0;
		dev->cache.initialize_ms	= 253;

		ret	= initialize_ioctl(dev, NULL);	// First
		_device_type_compute(dev);			// Must be first after init.
		_fifo_size_compute(dev);			// Must be second after init.
		_form_factor_compute(dev);			// Must be third after init.
		_auto_calibrate_compute(dev);
		_channels_compute(dev);
		_rate_generators_compute(dev);
		_sample_rates_compute(dev);
		_rate_divisors_compute(dev);
		_irq_mask_compute(dev);

		// Initialize additional resources.
		ret	= dev_irq_create(dev);
		if (ret)	break;

		ret	= dev_io_create(dev);
		if (ret)	break;

		dma	= GSC_DMA_SEL_STATIC
			| GSC_DMA_CAP_BMDMA_READ
			| (dev->cache.dmdma ? GSC_DMA_CAP_DMDMA_READ : 0);
		ret	= gsc_dma_create(dev, dma, dma);
		break;
	}

	return(ret);
}



/******************************************************************************
*
*	Function:	dev_device_destroy
*
*	Purpose:
*
*		Do everything needed to release the referenced device.
*
*	Arguments:
*
*		dev		The partial data for the device of interest.
*
*	Returned:
*
*		None.
*
******************************************************************************/

void dev_device_destroy(dev_data_t* dev)
{
	if (dev)
	{
		gsc_dma_destroy(dev);
		dev_io_destroy(dev);
		dev_irq_destroy(dev);
		gsc_bar_destroy(&dev->bar);
		os_sem_destroy(&dev->sem);
		os_spinlock_destroy(&dev->spinlock);
		os_pci_master_clear(&dev->pci);
		os_pci_dev_disable(&dev->pci);
	}
}


