// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16SDI/16SDI_Linux_4.x.x.x_GSC_DN/driver/irq.c $
// $Rev: 44505 $
// $Date: 2019-03-01 14:44:50 -0600 (Fri, 01 Mar 2019) $

// 16SDI: Device Driver: source file

#include "main.h"



//*****************************************************************************
void dev_irq_isr_local_handler(dev_data_t* dev)
{
	#define	BCR_IRQ_REQUEST	0x800

	u32	bcr;
	u32	irq;

	bcr	= os_reg_mem_rx_u32(NULL, dev->vaddr.gsc_bcr_32);

	if (bcr & BCR_IRQ_REQUEST)
	{
		// Clear the interrupt.
		bcr	^= BCR_IRQ_REQUEST;
		os_reg_mem_tx_u32(NULL, dev->vaddr.gsc_bcr_32, bcr);

		// Resume any waiting threads.
		irq	= GSC_FIELD_DECODE(bcr, 10, 8);

		switch (irq)
		{
			default:

				gsc_wait_resume_irq_main(dev, GSC_WAIT_MAIN_SPURIOUS);
				break;

			case SDI_IRQ_INIT_DONE:

				gsc_wait_resume_irq_gsc(dev, SDI_WAIT_GSC_INIT_DONE);
				break;

			case SDI_IRQ_AUTO_CAL_DONE:

				gsc_wait_resume_irq_gsc(dev, SDI_WAIT_GSC_AUTO_CAL_DONE);
				break;

			case SDI_IRQ_CHANNELS_READY:

				gsc_wait_resume_irq_gsc(dev, SDI_WAIT_GSC_CHANNELS_READY);
				break;

			case SDI_IRQ_BUF_THRESH_L2H:

				gsc_wait_resume_irq_gsc(dev, SDI_WAIT_GSC_BUF_THRESH_L2H);
				break;

			case SDI_IRQ_BUF_THRESH_H2L:

				gsc_wait_resume_irq_gsc(dev, SDI_WAIT_GSC_BUF_THRESH_H2L);
				break;

			case SDI_IRQ_BUF_AE:

				gsc_wait_resume_irq_gsc(dev, SDI_WAIT_GSC_BUF_AE);
				break;

			case SDI_IRQ_BUF_AF:

				gsc_wait_resume_irq_gsc(dev, SDI_WAIT_GSC_BUF_AF);
				break;
		}
	}
	else
	{
		// We don't know the source of the interrupt.
		gsc_wait_resume_irq_main(dev, GSC_WAIT_MAIN_SPURIOUS);
	}
}



//*****************************************************************************
int dev_irq_create(dev_data_t* dev)
{
	int	ret;

	os_reg_mem_tx_u32(dev, dev->vaddr.gsc_bcr_32, 0);
	ret	= gsc_irq_create(dev);
	return(ret);
}



//*****************************************************************************
void dev_irq_destroy(dev_data_t* dev)
{
	if (dev->vaddr.gsc_bcr_32)
		os_reg_mem_tx_u32(dev, dev->vaddr.gsc_bcr_32, 0);

	gsc_irq_destroy(dev);
}


