// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16SDI/16SDI_Linux_4.x.x.x_GSC_DN/driver/main.h $
// $Rev: 51954 $
// $Date: 2022-10-25 08:44:06 -0500 (Tue, 25 Oct 2022) $

// 16SDI: Device Driver: header file

#ifndef	__MAIN_H__
#define	__MAIN_H__

#include "16sdi.h"

#define	DEV_BAR_SHOW						0
#define	DEV_METRICS_SHOW					0
#define	DEV_PCI_ID_SHOW						0
#define	DEV_SUPPORTS_IRQ
#define	DEV_SUPPORTS_READ
#define	DEV_SUPPORTS_WAIT
#define	GSC_ALT_DEV_GET(a)					(a)
#define	GSC_ALT_STRUCT_GET(i,d)				(d)
#define	GSC_ALT_STRUCT_T					dev_data_t
#define	GSC_DEVS_PER_BOARD					1

typedef struct _dev_data_t					dev_data_t;

#include "gsc_main.h"



// macros *********************************************************************

#define	DEV_MODEL							"16SDI"	// Upper case form of the below.
#define	DEV_NAME							"16sdi"	// MUST AGREE WITH SDI_BASE_NAME

#define	DEV_VERSION							"5.6"	// FOR DEVICE SPECIFIC CODE ONLY!
// 5.6	Standardized various IOCTL system log messages.
//		Standardized processing of the initialization IOCTL service.
//		Standardized processing of the autocalibration IOCTL service.
//		Reorganization of ioctl.c.
//		Updated ioctl.c prototypes for consistency.
//		Removed unused macros.
//		Updated comments for consistency.
//		Made changes per file reorganization.
//		Made DMA portability changes.
// 5.5	Removed compiler warnings for Fedora 31.
//		Updated for porting to Windows.
//		Standardized various IOCTL system log messages.
//		Standardized processing of the initialization IOCTL service.
//		Standardized processing of the autocalibration IOCTL service.
// 5.4	Added support for the 5.x kernel series.
//		Increased I/O transfer buffer size.
// 5.3	Modified code so open succeeds even if initialization fails.
//		Updated per changes to common I/O code.
// 5.2	Bug fix: was ignoring BAR setup return value.
//		Updated per changes to the low level register interface.
//		Updated per changes to the OS specific PCI services.
//		Updated to use the newer common driver sources.
//		Made updates for DEV_SUPPORTS_XXX macro changes.
// 5.1	Made PCI registers accessible from inside an ISR.
//		The BAR code has been updated to include the data under a single structure.
//		The register definitions have been updated.
//		I/O services now pass around an os_mem_t structure instead of a buffer pointer.
//		White space cleanup.
//		Changed the arbitrary wait event callback argument to a void* type.
//		Modified to allow multiple apps to simultaneously access a single device.
//		Correct access to BAR0 and BAR1 (is RO, was RW).
//		Added an infinite I/O timeout option.
//		Corrected the IOCTL code for the register write and mod services.
//		Improved support for being signaled (i.e. being told to quit).
// 5.0	Updated to use the newer common driver sources.
//		Removed GNU notice from non-Linux specific files.
//		Removed Linux specific content from non-Linux specific source files.
//		White space cleanup.
//		Now using a spinlock rather than enabling and disabling interrupts.
//		Updated gsc_irq_open() and gsc_irq_close().
//		Updated gsc_dma_open() and gsc_dma_close().
//		Added three new IOCTL services.
// 4.5	Reduced #include list in driver interface header.
// 4.4	Updated for the 3.x kernel.
//		Corrected a system timer count roll over bug in ioctl.c.
// 4.3	BAR0 and BAR2 are now the only BAR regions used.
//		Include all common source, though not all are used.
// 4.2	Overhauled IOCTL based local interrupt implementation.
//		Updated per changes to the common source code.
//		Implemented IOCTL based I/O Abort support.
//		Implemented IOCTL based event/interrupt wait support.
//		Updated to use the common gsc_open() and gsc_close() services.
//		Implemented explicit local interrupt support.
//		Implemented support for the common PIO I/O routines.
//		Changed use of DEV_SUPPORTS_READ macro.
//		Changed use of DEV_SUPPORTS_WRITE macro.
//		Corrected the computed number of channels for the HS devices.
//		Removed compile errors under Fedora 7.
//		Eliminated the global dev_check_id() routine.
//		Updated the information that distinguishes PCI-16HSDI from PCI-16SDI-HS.
//		Added query option for the form factor.
//		Added query option for supported IRQ options.
// 4.1	Updated to the 2.6.29 kernel.
//		Made read services use common argument data types.
// 4.0	Overhauled driver.
// 3.3	Updated for the 2.6.25 kernel (Red Hat Fedora Core 9).
//		Updated for 64-bit kernel operation. Fixed the Sync Scan IOCTL service.
//		Added the Clear Buffer Sync IOCTL service. Fixed the "Wait For Ready"
//		code so that it didn't wait when the Channel Ready bit is already set.
//		Made some IOCTL services use device specfic features (# of RateGens ...).
// 3.1.5 Updated for the 2.6.21 kernel (Red Hat Fedora Core 7).

#ifndef	TRUE
	#define	TRUE							1
#endif

#ifndef	FALSE
	#define	FALSE							0
#endif

// I/O services
#define	DEV_IO_STREAM_QTY					(DEV_IO_RX_STREAM_QTY + DEV_IO_TX_STREAM_QTY)
#define	DEV_IO_RX_STREAM_QTY				1
#define	DEV_IO_TX_STREAM_QTY				0

#define	GSC_READ_PIO_WORK_32_BIT

// WAIT services
#define	DEV_WAIT_GSC_ALL					SDI_WAIT_GSC_ALL
#define	DEV_WAIT_ALT_ALL					SDI_WAIT_ALT_ALL
#define	DEV_WAIT_IO_ALL						SDI_WAIT_IO_ALL



// data types *****************************************************************

struct _dev_io_t
{
	// Initialized by open and IOCTL services (initialize and/or service specific).

	s32					io_mode;			// PIO, DMA, DMDMA
	s32					pio_threshold;		// Use PIO if samples <= this.
	s32					timeout_s;			// I/O timeout in seconds.

	// Initialized by I/O service at time of use.

	int					abort;
	gsc_dma_ch_t*		dma_channel;		// Use this channel for DMA.
	int					non_blocking;		// Is this non-blocking I/O?

	// Initialized by device specific I/O create code.

	int					bytes_per_sample;	// Sample size in bytes.
	u32					io_reg_offset;		// Offset of device's I/O FIFO.
	VADDR_T				io_reg_vaddr;		// Address of device's I/O FIFO.

	void				(*dev_io_sw_init)		(GSC_ALT_STRUCT_T* alt, dev_io_t* io);
	void				(*dev_io_close)			(GSC_ALT_STRUCT_T* alt, dev_io_t* io);
	void				(*dev_io_open)			(GSC_ALT_STRUCT_T* alt, dev_io_t* io);
	int					(*dev_io_startup)		(GSC_ALT_STRUCT_T* alt, dev_io_t* io);
	long				(*dev_pio_available)	(GSC_ALT_STRUCT_T* alt, dev_io_t* io, size_t count);
	long				(*dev_bmdma_available)	(GSC_ALT_STRUCT_T* alt, dev_io_t* io, size_t count);
	long				(*dev_dmdma_available)	(GSC_ALT_STRUCT_T* alt, dev_io_t* io, size_t count);
	long				(*dev_pio_xfer)			(GSC_ALT_STRUCT_T* alt, dev_io_t* io, const os_mem_t* mem, size_t count, os_time_tick_t st_end);
	long				(*dev_bmdma_xfer)		(GSC_ALT_STRUCT_T* alt, dev_io_t* io, const os_mem_t* mem, size_t count, os_time_tick_t st_end);
	long				(*dev_dmdma_xfer)		(GSC_ALT_STRUCT_T* alt, dev_io_t* io, const os_mem_t* mem, size_t count, os_time_tick_t st_end);

	struct
	{
		u32				abort;		// ...WAIT_IO_XXX_ABORT
		u32				done;		// ...WAIT_IO_XXX_DONE
		u32				error;		// ...WAIT_IO_XXX_ERROR
		u32				timeout;	// ...WAIT_IO_XXX_TIMEOUT
	} wait;

	// Initialize by GSC commone create code.

	os_mem_t			mem;				// I/O buffer.
	os_sem_t			sem;				// Only one Tx or Rx at a time.
};

struct _dev_data_t
{
	os_pci_t			pci;			// The kernel PCI device descriptor.
	os_data_t			os;				// OS specific data.
	os_spinlock_t		spinlock;		// Control ISR access.
	os_sem_t			sem;			// Control thread access.
	gsc_dev_type_t		board_type;		// Corresponds to basic device type.
	const char*			model;			// Base model number in upper case.
	int					board_index;	// Index of the device being accessed.
	int					users;			// Number of currently active open() requests.
	int					share;			// Were we opened in shared mode?

	gsc_bar_t			bar;			// device register mappings
	gsc_dma_t			dma;			// For DMA based I/O.
	gsc_irq_t			irq;			// For interrut support.
	gsc_wait_node_t*	wait_list;

	struct
	{					// This is for streaming I/O
		dev_io_t		rx;			// Analog Input read
		dev_io_t*		io_streams[DEV_IO_STREAM_QTY];
	} io;

	struct
	{
		VADDR_T			plx_intcsr_32;	// Interrupt Control/Status Register
		VADDR_T			plx_dmaarb_32;	// DMA Arbitration Register
		VADDR_T			plx_dmathr_32;	// DMA Threshold Register

		VADDR_T			gsc_bcr_32;		// 0x00 Board Control Register
		VADDR_T			gsc_rcar_32;	// 0x04 Rate Control A Register
		VADDR_T			gsc_rcbr_32;	// 0x08 Rate Control B Register
		VADDR_T			gsc_rccr_32;	// 0x0C Rate Control C Register
		VADDR_T			gsc_rcdr_32;	// 0x10 Rate Control D Register
		VADDR_T			gsc_rar_32;		// 0x14 Rate Assignment Register
		VADDR_T			gsc_rd01r_32;	// 0x18 Rate Divisor Channels 0/1 Register
		VADDR_T			gsc_rd23r_32;	// 0x1C Rate Divisor Channels 2/3 Register
		VADDR_T			gsc_rd45r_32;	// 0x20 Rate Divisor Channels 4/5 Register
		VADDR_T			gsc_rd67r_32;	// 0x24 Rate Divisor Channels 6/7 Register
		VADDR_T			gsc_rd89r_32;	// 0x28 Rate Divisor Channels 8/9 Register
		VADDR_T			gsc_rdabr_32;	// 0x2C Rate Divisor Channels 10/11 Register
		VADDR_T			gsc_rdcdr_32;	// 0x30 Rate Divisor Channels 12/13 Register
		VADDR_T			gsc_rdefr_32;	// 0x34 Rate Divisor Channels 14/15 Register
		VADDR_T			gsc_btr_32;		// 0x38 Buffer Threshold Register
		VADDR_T			gsc_brr_32;		// 0x3C Board Revision Register
		VADDR_T			gsc_bsr_32;		// 0x40 Buffer Size Register
										// 0x44
		VADDR_T			gsc_idbr_32;	// 0x48 Input Data Buffer Register
	} vaddr;

	struct
	{
		int				auto_cal_ms;		// Maximum ms for auto-cal
		int				initialize_ms;		// Maximum ms for initialize

		u32				gsc_brr_32;			// Board Revision Register
		int				brr_chan_d16;		// Channel count if BRR D16 is set.
		int				brr_chan_d17;		// Channel count if BRR D17 is set.

		int				channel_qty;		// The number of channels on the device.
		int				channels_max;		// Maximum channels supported by model.
		int				channel_groups;		// The number of channel groups on the device.

		int				dmdma;				// Is DMDMA supported?
		u32				fifo_size;			// Size of FIFO - not the fill level.
		s32				form_factor;		// PCI, PMC ...
		u32				threshold_mask;		// Mask of valid Buf Thresh Reg bits.

		s32				irq_mask;			// A 1 for each supported IRQ option.

		u32				rate_div_ndiv_mask;	// Mask of valid bits.
		int				rate_div_ndiv_max;	// Minimum Nrate value.
		int				rate_div_ndiv_min;	// Maximum Nrate value.

		int				rate_gen_fgen_max;	// Rate Generator maximum output rate.
		int				rate_gen_fgen_min;	// Rate Generator minimum output rate.
		int				rate_gen_nrate_mask;// Mask of valid bits.
		int				rate_gen_nrate_max;	// Minimum Nrate value.
		int				rate_gen_nrate_min;	// Maximum Nrate value.
		int				rate_gen_qty;		// The number of rate generators on the device.

		u32				fsamp_max;			// The maximum sample rate supported.
		u32				fsamp_min;			// The minimum sample rate supported.
	} cache;
};



// prototypes *****************************************************************

void		dev_io_close(dev_data_t* dev);
int			dev_io_create(dev_data_t* dev);
void		dev_io_destroy(dev_data_t* dev);
int			dev_io_open(dev_data_t* dev);
dev_io_t*	dev_io_read_select(dev_data_t* dev, size_t count);
int			dev_irq_create(dev_data_t* dev);
void		dev_irq_destroy(dev_data_t* dev);
int			dev_read_create(dev_data_t* dev, dev_io_t* io);

int			initialize_ioctl(dev_data_t* dev, void* arg);



#endif
