// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16SDI/16SDI_Linux_4.x.x.x_GSC_DN/samples/billion/perform.c $
// $Rev: 51945 $
// $Date: 2022-10-25 08:31:07 -0500 (Tue, 25 Oct 2022) $

// 16SDI: Sample Application: source file

#include "main.h"



// macros *********************************************************************

#define	_1B		(1000L * 1000L * 1000L)



// variables ******************************************************************

static	u32	_buffer[THRESHOLD];



//*****************************************************************************
static int _read_data(const args_t* args)
{
	long	bytes	= 0;
	int		calls	= 0;
	int		errs	= 0;
	long	get;
	long	got;

	gsc_label("Reading");

	for (;;)
	{
		get	= _1B - bytes;

		if (get > sizeof(_buffer))
			get	= sizeof(_buffer);

		calls++;
		got	= sdi_read(args->fd, _buffer, get);

		if (got < 0)
		{
			errs	= 1;
			printf(	"FAIL <---  (sdi_read error, error %d)\n", (int) got);
			break;
		}

		if (got != get)
		{
			errs	= 1;
			printf("FAIL <---  (total ");
			gsc_label_long_comma(bytes);
			printf(" bytes, requested ");
			gsc_label_long_comma(get);
			printf(", got ");
			gsc_label_long_comma(got);
			printf(", made ");
			gsc_label_long_comma(calls);
			printf(" call%s)\n", (calls == 1) ? "" : "s");
			break;
		}

		bytes	+= got;

		if (bytes >= _1B)
			break;
	}

	if (errs == 0)
	{
		printf("PASS  (sdi_read ");
		gsc_label_long_comma(bytes);
		printf(" bytes via ");
		gsc_label_long_comma(calls);
		printf(" call%s)\n", (calls == 1) ? "" : "s");
	}

	return(errs);
}



//*****************************************************************************
int perform_tests(const args_t* args)
{
	int	errs	= 0;
	s32	fsamp	= 1100000L;	// highest possible
	s32	ndiv;
	s32	nrate;

	errs	+= sdi_rx_io_mode			(args->fd, -1, 1, GSC_IO_MODE_BMDMA, NULL);
	errs	+= sdi_rx_io_timeout		(args->fd, -1, 1, 20, NULL);
	errs	+= sdi_initialize			(args->fd, -1, 1);
	errs	+= sdi_ain_mode				(args->fd, -1, 1, SDI_AIN_MODE_DIFF, NULL);
	errs	+= sdi_ain_range			(args->fd, -1, 1, SDI_AIN_RANGE_10V, NULL);
	errs	+= sdi_ain_buf_input		(args->fd, -1, 1, SDI_AIN_BUF_INPUT_ENABLE, NULL);
	errs	+= sdi_ain_buf_thresh		(args->fd, -1, 1, THRESHOLD, NULL);
	errs	+= sdi_ch_grp_src_all		(args->fd, -1, 1, SDI_CH_GRP_SRC_GEN_A);
	errs	+= sdi_channel_order		(args->fd, -1, 1, SDI_CHANNEL_ORDER_SYNC, NULL);
	errs	+= sdi_data_format			(args->fd, -1, 1, SDI_DATA_FORMAT_OFF_BIN, NULL);
	errs	+= sdi_init_mode			(args->fd, -1, 1, SDI_INIT_MODE_INITIATOR, NULL);
	errs	+= sdi_fsamp_ai_compute		(args->fd, -1, 1, fsamp, &nrate, &ndiv, NULL);
	errs	+= sdi_rate_div_ndiv_all	(args->fd, -1, 1, ndiv);
	errs	+= sdi_rate_gen_nrate_all	(args->fd, -1, 1, nrate);
	errs	+= sdi_fsamp_ai_report_all	(args->fd, -1, 1, NULL);
	errs	+= sdi_auto_calibrate		(args->fd, -1, 1);
	errs	+= sdi_ain_buf_clear		(args->fd, -1, 1);

	errs	+= _read_data(args);
	return(errs);
}


