// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16SDI/16SDI_Linux_4.x.x.x_GSC_DN/samples/rxrate/perform.c $
// $Rev: 50368 $
// $Date: 2022-02-15 16:56:43 -0600 (Tue, 15 Feb 2022) $

// 16SDI: Sample Application: source file

#include "main.h"



// variables ******************************************************************

static u32	_rx[THRESHOLD];



//*****************************************************************************
static int _rate_test(const args_t* args, long get)
{
	struct timeval	begin;
	int				errs	= 0;
	long			got;
	long long		limit	= (long long) args->rx_mb * 1000000L;
	struct timeval	minimum;
	struct timeval	now;
	long long		sps;
	long long		total	= 0;
	long			us		= 1000000L * args->seconds;

	gsc_label("Reading");

	gettimeofday(&begin, NULL);
	minimum.tv_sec	= begin.tv_sec + args->seconds;
	minimum.tv_usec	= begin.tv_usec;

	for (;;)
	{
		got	= sdi_read(args->fd, _rx, get);

		if (got < 0)
		{
			errs++;
			printf("FAIL <--- (read");
			printf(", total so far %lld", total);
			printf(", requested %ld bytes", (long) get);
			printf(", error %d)\n", (int) got);
			break;
		}

		gettimeofday(&now, NULL);
		total	+= got;

		if ((total >= limit) && (now.tv_sec > minimum.tv_sec))
		{
			us	= (now.tv_sec - begin.tv_sec) * 1000000L
				+ (now.tv_usec - begin.tv_usec);
			sps	= (total * 10000000L / 4 / us + 5) / 10;

			printf(	"PASS  (");
			gsc_label_long_comma(total);
			printf(" Bytes, %ld.%06ld Seconds, ", us / 1000000, us % 1000000);
			gsc_label_long_comma(sps);
			printf(" S/S)\n");
			break;
		}
	}

	return(errs);
}



//*****************************************************************************
int perform_tests(const args_t* args)
{
	int		errs	= 0;
	s32		fsamp	= 1100000L;	// highest possible
	long	get		= sizeof(_rx);
	s32		ndiv;
	s32		nrate;

	errs	+= sdi_initialize			(args->fd, -1, 1);
	errs	+= sdi_rx_io_mode			(args->fd, -1, 1, args->io_mode, NULL);
	errs	+= sdi_rx_io_timeout		(args->fd, -1, 1, 20, NULL);
	errs	+= sdi_ain_mode				(args->fd, -1, 1, SDI_AIN_MODE_DIFF, NULL);
	errs	+= sdi_ain_range			(args->fd, -1, 1, SDI_AIN_RANGE_10V, NULL);
	errs	+= sdi_ain_buf_input		(args->fd, -1, 1, SDI_AIN_BUF_INPUT_ENABLE, NULL);
	errs	+= sdi_ain_buf_thresh		(args->fd, -1, 1, THRESHOLD, NULL);
	errs	+= sdi_ch_grp_src_all		(args->fd, -1, 1, SDI_CH_GRP_SRC_GEN_A);
	errs	+= sdi_channel_order		(args->fd, -1, 1, SDI_CHANNEL_ORDER_SYNC, NULL);
	errs	+= sdi_data_format			(args->fd, -1, 1, SDI_DATA_FORMAT_OFF_BIN, NULL);
	errs	+= sdi_init_mode			(args->fd, -1, 1, SDI_INIT_MODE_INITIATOR, NULL);
	errs	+= sdi_fsamp_ai_compute		(args->fd, -1, 1, fsamp, &nrate, &ndiv, NULL);
	errs	+= sdi_rate_div_ndiv_all	(args->fd, -1, 1, ndiv);
	errs	+= sdi_rate_gen_nrate_all	(args->fd, -1, 1, nrate);
	errs	+= sdi_fsamp_ai_report_all	(args->fd, -1, 1, NULL);
	errs	+= sdi_auto_calibrate		(args->fd, -1, 1);

	gsc_label ("Read Rate");
	printf("(%d second minimum, %d MB minimum)\n", (int) args->seconds, (int) args->rx_mb);
	gsc_label_level_inc();

	errs	+= sdi_ain_buf_clear(args->fd, -1, 1);

	errs	+= _rate_test(args, get);

	gsc_label_level_dec();
	return(errs);
}



