// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16SDI/16SDI_Linux_4.x.x.x_GSC_DN/samples/savedata/perform.c $
// $Rev: 50371 $
// $Date: 2022-02-15 16:57:50 -0600 (Tue, 15 Feb 2022) $

// 16SDI: Sample Application: source file

#include "main.h"



// macros *********************************************************************

#define	_1M		(1024L * 1024L)



// variables ******************************************************************

static	u32	_buffer[_1M];



//*****************************************************************************
static int _channels(const args_t* args, s32* chans)
{
	int	errs;

	gsc_label("Input Channels");
	errs	= sdi_query(args->fd, -1, 0, SDI_QUERY_CHANNEL_QTY, chans);

	if (errs == 0)
		printf("%d Channels\n", chans[0]);

	return(errs);
}



//*****************************************************************************
static int _read_data(const args_t* args)
{
	int		errs;
	long	get		= sizeof(_buffer);
	int		got;

	gsc_label("Reading");
	got	= sdi_read(args->fd, _buffer, get);

	if (got < 0)
	{
		errs	= 1;
		printf(	"FAIL <---  (error %d)\n", got);
	}
	else if (got != get)
	{
		errs	= 1;
		printf(	"FAIL <---  (got %ld bytes, requested %ld)\n",
				(long) got,
				(long) get);
	}
	else
	{
		errs	= 0;
		printf(	"PASS  (%ld bytes)\n",
				(long) get);
	}

	return(errs);
}



//*****************************************************************************
static int _save_data(int chan_qty, int errs)
{
	FILE*		file;
	int			i;
	long		l;
	const char*	name	= "data.txt";
	long		samples	= sizeof(_buffer) / 4;

	gsc_label("Saving");

	for (;;)
	{
		if (errs)
		{
			printf("SKIPPED  (errors)\n");
			errs	= 0;
			break;
		}

		file	= fopen(name, "w+b");

		if (file == NULL)
		{
			printf("FAIL <---  (unable to create %s)\n", name);
			errs	= 1;
			break;
		}

		for (l = 0; l < samples; l++)
		{
			i	= fprintf(file, "  %08lX", (long) _buffer[l]);

			if (i != 10)
			{
				printf("FAIL <---  (fprintf() failure to %s)\n", name);
				errs	= 1;
				break;
			}

			if ((l % chan_qty) == (chan_qty - 1))
			{
				i	= fprintf(file, "\r\n");

				if (i != 2)
				{
					printf("FAIL <---  (fprintf() failure to %s)\n", name);;
					errs	= 1;
					break;
				}
			}
		}

		fclose(file);

		if (errs == 0)
			printf("PASS  (%s)\n", name);

		break;
	}

	return(errs);
}



//*****************************************************************************
int perform_tests(const args_t* args)
{
	int	chan_qty;
	int	errs		= 0;

	gsc_label("Capture & Save");
	printf("\n");
	gsc_label_level_inc();

	errs	+= sdi_config_ai(args->fd, -1, 1, 100000);
	errs	+= sdi_rx_io_mode(args->fd, -1, 1, args->io_mode, NULL);
	errs	+= _channels(args, &chan_qty);
	errs	+= sdi_ain_buf_clear(args->fd, -1, 1);
	errs	+= _read_data(args);
	errs	+= _save_data(chan_qty, errs);

	gsc_label_level_dec();
	return(errs);
}



