// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16SDI/16SDI_Linux_4.x.x.x_GSC_DN/samples/sbtest/channel_group.c $
// $Rev: 51952 $
// $Date: 2022-10-25 08:36:47 -0500 (Tue, 25 Oct 2022) $

// 16SDI: Sample Application: source file

#include "main.h"



/******************************************************************************
*
*	Function:	_channel_group_source_test
*
*	Purpose:
*
*		Perform a test of the IOCTL service SDI_IOCTL_CH_GRP_X_SRC.
*
*	Arguments:
*
*		fd		The handle for the device to access.
*
*		cmd		The IOCTL command code to test.
*
*		grp		The index of the channel group to access.
*
*		src		The source to test.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

static int _channel_group_source_test(int fd, long cmd, int grp, int src)
{
	int	errs;
	int	gen		= 0;
	s32	gens;
	u32	mask;
	int	shift;
	s32	val		= 0;

	shift	= 4 * grp;
	mask	= 0xF << shift;
	errs	= sdi_query(fd, -1, 0, SDI_QUERY_RATE_GEN_QTY, &gens);

	switch (src)
	{
		default:

			errs	= 1;
			printf(	"FAIL <---  (%d. INTERNAL ERROR, src 0x%lX)\n",
					__LINE__,
					(long) src);
			break;

		case SDI_CH_GRP_SRC_GEN_A:		gen	= 0;	val	= 0;	break;
		case SDI_CH_GRP_SRC_GEN_B:		gen	= 1;	val	= 1;	break;
		case SDI_CH_GRP_SRC_GEN_C:		gen	= 2;	val	= 2;	break;
		case SDI_CH_GRP_SRC_GEN_D:		gen	= 3;	val	= 3;	break;
		case SDI_CH_GRP_SRC_EXTERN:		gen	= 0;	val	= 4;	break;
		case SDI_CH_GRP_SRC_DISABLE:	gen	= 0;	val	= 5;	break;
	}

	if (errs)
	{
		// There was a problem.
	}
	else  if (gen >= gens)
	{
		// This source isn't supported on this device.
	}
	else
	{
		service_data_t	list[]	=
		{
			{
				/* service	*/	SERVICE_IOCTL_SET,
				/* cmd		*/	SDI_IOCTL_INITIALIZE,
				/* arg		*/	0,
				/* reg		*/	-1,
				/* mask		*/	0,
				/* value	*/	0
			},

			{				// DISABLE then selection
				/* service	*/	SERVICE_NORMAL,
				/* cmd		*/	cmd,
				/* arg		*/	SDI_CH_GRP_SRC_DISABLE,
				/* reg		*/	SDI_GSC_RAR,
				/* mask		*/	mask,
				/* value	*/	5 << shift
			},
			{
				/* service	*/	SERVICE_NORMAL,
				/* cmd		*/	cmd,
				/* arg		*/	val,
				/* reg		*/	SDI_GSC_RAR,
				/* mask		*/	mask,
				/* value	*/	val << shift
			},

			{				// EXTERNAL then selection
				/* service	*/	SERVICE_NORMAL,
				/* cmd		*/	cmd,
				/* arg		*/	SDI_CH_GRP_SRC_EXTERN,
				/* reg		*/	SDI_GSC_RAR,
				/* mask		*/	mask,
				/* value	*/	4 << shift
			},
			{
				/* service	*/	SERVICE_NORMAL,
				/* cmd		*/	cmd,
				/* arg		*/	val,
				/* reg		*/	SDI_GSC_RAR,
				/* mask		*/	mask,
				/* value	*/	val << shift
			},

			{				// GEN B then selection
				/* service	*/	SERVICE_NORMAL,
				/* cmd		*/	cmd,
				/* arg		*/	SDI_CH_GRP_SRC_GEN_B,
				/* reg		*/	SDI_GSC_RAR,
				/* mask		*/	mask,
				/* value	*/	1 << shift
			},
			{
				/* service	*/	SERVICE_NORMAL,
				/* cmd		*/	cmd,
				/* arg		*/	val,
				/* reg		*/	SDI_GSC_RAR,
				/* mask		*/	mask,
				/* value	*/	val << shift
			},

			{				// GEN A then selection
				/* service	*/	SERVICE_NORMAL,
				/* cmd		*/	cmd,
				/* arg		*/	SDI_CH_GRP_SRC_GEN_A,
				/* reg		*/	SDI_GSC_RAR,
				/* mask		*/	mask,
				/* value	*/	0 << shift
			},
			{
				/* service	*/	SERVICE_NORMAL,
				/* cmd		*/	cmd,
				/* arg		*/	val,
				/* reg		*/	SDI_GSC_RAR,
				/* mask		*/	mask,
				/* value	*/	val << shift
			},

			{
				/* service	*/	SERVICE_IOCTL_SET,
				/* cmd		*/	SDI_IOCTL_INITIALIZE,
				/* arg		*/	0,
				/* reg		*/	-1,
				/* mask		*/	0,
				/* value	*/	0
			},

			{ SERVICE_END_LIST }
		};

		errs	= service_ioctl_set_reg_list(fd, list);

		if (errs == 0)
			errs	= service_ioctl_reg_get_list(fd, list);
	}

	return(errs);
}



/******************************************************************************
*
*	Function:	_channel_group_tests
*
*	Purpose:
*
*		Perform a test of the IOCTL service SDI_IOCTL_CH_GRP_X_SRC.
*
*	Arguments:
*
*		fd		The handle for the device to access.
*
*		cmd		The IOCTL command code to test.
*
*		name	A text version of the above command code.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

static int _channel_group_tests(int fd, long cmd, const char* name)
{
	int	errs	= 0;
	int	grp		= 0;
	s32	grps	= 0;

	for (;;)	// A convenience loop.
	{
		gsc_label(name);

		switch (cmd)
		{
			default:

				errs	= 1;
				printf(	"FAIL <---  (%d. INTERNAL ERROR, cmd 0x%lX)\n",
						__LINE__,
						(long) cmd);
				break;

			case SDI_IOCTL_CH_GRP_0_SRC:	grp	= 0;	break;
			case SDI_IOCTL_CH_GRP_1_SRC:	grp	= 1;	break;
			case SDI_IOCTL_CH_GRP_2_SRC:	grp	= 2;	break;
			case SDI_IOCTL_CH_GRP_3_SRC:	grp	= 3;	break;
		}

		if (errs == 0)
			errs	= sdi_query(fd, -1, 0, SDI_QUERY_CHANNEL_GPS, &grps);

		if ((errs == 0) && (grp >= grps))
		{
			printf("SKIPPED  (This isn't supported on this device.)\n");
			break;
		}

		if (errs == 0)
			errs	= _channel_group_source_test(fd, cmd, grp, SDI_CH_GRP_SRC_GEN_A);

		if (errs == 0)
			errs	= _channel_group_source_test(fd, cmd, grp, SDI_CH_GRP_SRC_GEN_B);

		if (errs == 0)
			errs	= _channel_group_source_test(fd, cmd, grp, SDI_CH_GRP_SRC_GEN_C);

		if (errs == 0)
			errs	= _channel_group_source_test(fd, cmd, grp, SDI_CH_GRP_SRC_GEN_D);

		if (errs == 0)
			errs	= _channel_group_source_test(fd, cmd, grp, SDI_CH_GRP_SRC_EXTERN);

		if (errs == 0)
			errs	= _channel_group_source_test(fd, cmd, grp, SDI_CH_GRP_SRC_DISABLE);

		if (errs == 0)
			printf("PASS\n");

		break;
	}

	return(errs);
}



/******************************************************************************
*
*	Function:	channel_group_0_source_test
*
*	Purpose:
*
*		Perform a test of the IOCTL service SDI_IOCTL_CH_GRP_0_SRC.
*
*	Arguments:
*
*		fd		The handle for the device to access.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int channel_group_0_source_test(int fd)
{
	int	errs;

	errs	= _channel_group_tests(
				fd,
				SDI_IOCTL_CH_GRP_0_SRC,
				"SDI_IOCTL_CH_GRP_0_SRC");
	return(errs);
}



/******************************************************************************
*
*	Function:	channel_group_1_source_test
*
*	Purpose:
*
*		Perform a test of the IOCTL service SDI_IOCTL_CH_GRP_1_SRC.
*
*	Arguments:
*
*		fd		The handle for the device to access.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int channel_group_1_source_test(int fd)
{
	int	errs;

	errs	= _channel_group_tests(
				fd,
				SDI_IOCTL_CH_GRP_1_SRC,
				"SDI_IOCTL_CH_GRP_1_SRC");
	return(errs);
}


/******************************************************************************
*
*	Function:	channel_group_2_source_test
*
*	Purpose:
*
*		Perform a test of the IOCTL service SDI_IOCTL_CH_GRP_2_SRC.
*
*	Arguments:
*
*		fd		The handle for the device to access.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int channel_group_2_source_test(int fd)
{
	int	errs;

	errs	= _channel_group_tests(
				fd,
				SDI_IOCTL_CH_GRP_2_SRC,
				"SDI_IOCTL_CH_GRP_2_SRC");
	return(errs);
}


/******************************************************************************
*
*	Function:	channel_group_3_source_test
*
*	Purpose:
*
*		Perform a test of the IOCTL service SDI_IOCTL_CH_GRP_3_SRC.
*
*	Arguments:
*
*		fd		The handle for the device to access.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int channel_group_3_source_test(int fd)
{
	int	errs;

	errs	= _channel_group_tests(
				fd,
				SDI_IOCTL_CH_GRP_3_SRC,
				"SDI_IOCTL_CH_GRP_3_SRC");
	return(errs);
}


