// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16SDI/16SDI_Linux_4.x.x.x_GSC_DN/samples/sbtest/reg_mod.c $
// $Rev: 51952 $
// $Date: 2022-10-25 08:36:47 -0500 (Tue, 25 Oct 2022) $

// 16SDI: Sample Application: source file

#include "main.h"



/******************************************************************************
*
*	Function:	reg_mod_test
*
*	Purpose:
*
*		Perform a test of the IOCTL service SDI_IOCTL_REG_MOD.
*
*	Arguments:
*
*		fd		The handle for the device to access.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int reg_mod_test(int fd)
{
	static const struct
	{
		u32	reg;
		u32	mask;
	} list[]	=
	{
		// reg					mask
		{ SDI_GSC_BCR,			0x0000073F	},
		{ SDI_GSC_RAR,			0x000000FF	},
		{ SDI_GSC_RD01R,		0x0000003F	},
		{ SDI_GSC_RD23R,		0x0000003F	},
		{ SDI_GSC_RD45R,		0x0000003F	},
		{ SDI_GSC_BTR,			0x0000FFFF	},
	};

	u32			bit;
	int			errs	= 0;
	int			i;
	gsc_reg_t	parm;
	int			ret;
	u32			value;

	gsc_label("SDI_IOCTL_REG_MOD");

	for (i = 0; (errs == 0) && (i < SIZEOF_ARRAY(list)); i++)
	{

		for (bit = 0x1; bit; bit <<= 1)
		{
			if ((list[i].mask & bit) == 0)
				continue;

			// Read the register value so we can restore it later.
			parm.reg	= list[i].reg;
			ret			= sdi_ioctl(fd, SDI_IOCTL_REG_READ, (void*) &parm);
			value		= parm.value;

			if (ret)
			{
				errs	= 1;
				printf(	"FAIL <---  (%d. i %d, sdi_ioctl() failure, error %d)\n",
						__LINE__,
						i,
						ret);
				break;
			}

			// Set the bit low.
			parm.value	= 0;
			parm.mask	= bit;
			ret			= sdi_ioctl(fd, SDI_IOCTL_REG_MOD, (void*) &parm);

			if (ret)
			{
				errs	= 1;
				printf(	"FAIL <---  (%d. i %d, sdi_ioctl() failure, error %d)\n",
						__LINE__,
						i,
						ret);
				break;
			}

			// Now verify that the bit is low.
			ret	= sdi_ioctl(fd, SDI_IOCTL_REG_READ, (void*) &parm);

			if (ret)
			{
				errs	= 1;
				printf(	"FAIL <---  (%d. i %d, sdi_ioctl() failure, error %d)\n",
						__LINE__,
						i,
						ret);
				break;
			}

			if (parm.value & bit)
			{
				errs	= 1;
				printf(	"FAIL <---  (%d. i %d, bit not low, mask 0x%lX, got 0x%lX)\n",
						__LINE__,
						i,
						(long) bit,
						(long) parm.value);
				break;
			}

			// Now restore the register to its previous value.
			parm.value	= value;
			ret			= sdi_ioctl(fd, SDI_IOCTL_REG_WRITE, (void*) &parm);

			if (ret)
			{
				errs	= 1;
				printf(	"FAIL <---  (%d. i %d, sdi_ioctl() failure, error %d)\n",
						__LINE__,
						i,
						ret);
				break;
			}

			// Set the bit high.
			parm.value	= bit;
			parm.mask	= bit;
			ret			= sdi_ioctl(fd, SDI_IOCTL_REG_MOD, (void*) &parm);

			if (ret)
			{
				errs	= 1;
				printf(	"FAIL <---  (%d. i %d, sdi_ioctl() failure, error %d)\n",
						__LINE__,
						i,
						ret);
				break;
			}

			// Now verify that the bit is high.
			ret	= sdi_ioctl(fd, SDI_IOCTL_REG_READ, (void*) &parm);

			if (ret)
			{
				errs	= 1;
				printf(	"FAIL <---  (%d. i %d, sdi_ioctl() failure, error %d)\n",
						__LINE__,
						i,
						ret);
				break;
			}

			if ((parm.value & bit) == 0)
			{
				errs	= 1;
				printf(	"FAIL <---  (%d. i %d, bit not high, mask 0x%lX, got 0x%lX)\n",
						__LINE__,
						i,
						(long) bit,
						(long) parm.value);
				break;
			}

			// Now restore the register to its previous value.
			parm.value	= value;
			ret			= sdi_ioctl(fd, SDI_IOCTL_REG_WRITE, (void*) &parm);

			if (ret)
			{
				errs	= 1;
				printf(	"FAIL <---  (%d. i %d, sdi_ioctl() failure, error %d)\n",
						__LINE__,
						i,
						ret);
				break;
			}
		}
	}

	if (errs == 0)
		printf("PASS\n");

	return(errs);
}


