// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16SDI/16SDI_Linux_4.x.x.x_GSC_DN/samples/sbtest/reg_write.c $
// $Rev: 51952 $
// $Date: 2022-10-25 08:36:47 -0500 (Tue, 25 Oct 2022) $

// 16SDI: Sample Application: source file

#include "main.h"



/******************************************************************************
*
*	Function:	reg_write_test
*
*	Purpose:
*
*		Perform a test of the IOCTL service SDI_IOCTL_REG_WRITE.
*
*	Arguments:
*
*		fd		The handle for the device to access.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int reg_write_test(int fd)
{
	static const struct
	{
		u32	reg;
		u32	value;
		u32	mask;
	} list[]	=
	{
		// reg					value		mask
		{ SDI_GSC_BCR,		0x0000033F,	0x0000073F	},
		{ SDI_GSC_BCR,		0x00000000,	0x0000073F	},

		{ SDI_GSC_RAR,		0x00000022,	0x000000FF	},
		{ SDI_GSC_RAR,		0x00000000,	0x000000FF	},

		{ SDI_GSC_RD01R,	0x00000011,	0x0000003F	},
		{ SDI_GSC_RD01R,	0x00000005,	0x0000003F	},

		{ SDI_GSC_RD23R,	0x00000011,	0x0000003F	},
		{ SDI_GSC_RD23R,	0x00000005,	0x0000003F	},

		{ SDI_GSC_RD45R,	0x00000011,	0x0000003F	},
		{ SDI_GSC_RD45R,	0x00000005,	0x0000003F	},

		{ SDI_GSC_BTR,		0x00009999,	0x0000FFFF	},
		{ SDI_GSC_BTR,		0x0003FFFE,	0x000CFFFF	},
	};

	int			errs	= 0;
	int			i;
	gsc_reg_t	parm;
	int			ret;

	gsc_label("SDI_IOCTL_REG_WRITE");

	for (i = 0; i < SIZEOF_ARRAY(list); i++)
	{
		parm.reg	= list[i].reg;
		parm.value	= list[i].value;
		parm.mask	= 0xBEEFDEAD;
		ret			= sdi_ioctl(fd, SDI_IOCTL_REG_WRITE, (void*) &parm);

		if (ret)
		{
			errs	= 1;
			printf(	"FAIL <---  (%d. i %d, sdi_ioctl() failure, error %d)\n",
					__LINE__,
					i,
					ret);
			break;
		}

		if (parm.mask != 0xBEEFDEAD)
		{
			errs	= 1;
			printf("FAIL <---  (%d. i %d, mask changed)\n", __LINE__, i);
			break;
		}

		// Now read the register and verify the bits of interest.
		ret	= sdi_ioctl(fd, SDI_IOCTL_REG_READ, (void*) &parm);

		if (ret)
		{
			errs	= 1;
			printf(	"FAIL <---  (%d. i %d, sdi_ioctl() failure, error %d)\n",
					__LINE__,
					i,
					ret);
			break;
		}

		if ((list[i].value & list[i].mask) != (parm.value & list[i].mask))
		{
			errs	= 1;
			printf(	"FAIL <---  (%d. i %d, mask 0x%lX, got 0x%lX, expected 0x%lX)\n",
					__LINE__,
					i,
					(long) list[i].mask,
					(long) parm.value,
					(long) list[i].value);
			break;
		}
	}

	if (errs == 0)
		printf("PASS\n");

	return(errs);
}


