// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16SDI/16SDI_Linux_4.x.x.x_GSC_DN/samples/sbtest/voltage.c $
// $Rev: 51952 $
// $Date: 2022-10-25 08:36:47 -0500 (Tue, 25 Oct 2022) $

// 16SDI: Sample Application: source file

#include "main.h"



// variables ******************************************************************

static u32	_data[4096];



/******************************************************************************
*
*	Function:	_v_test
*
*	Purpose:
*
*		Perform ADC voltage accuracy tests.
*
*	Arguments:
*
*		fd			The handle for the device to access.
*
*		title		The title to use for this test.
*
*		title		The title to use for this test.
*
*		ain_mode	The analog input mode to select for the test.
*
*		ain_range	The input range to select for the test.
*
*		range		The positive full scale voltage range.
*
*		acc			The expected accuracy.
*
*		frac		The fraction of the full range to test for.
*
*		target		The target voltage reading expected.
*
*		delta		The tolerable reading varience.
*
*	Returned:
*
*		>= 0		The number of errors encounterred.
*
******************************************************************************/

static int _v_test(
	int			fd,
	const char*	title,
	s32			ain_mode,
	s32			ain_range,
	float		range,
	float		acc,
	float		frac)
{
	service_data_t	list_1[]	=
	{
		{
			/* service	*/	SERVICE_IOCTL_SET,
			/* cmd		*/	SDI_IOCTL_INITIALIZE,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	SDI_IOCTL_RX_IO_MODE,
			/* arg		*/	GSC_IO_MODE_PIO,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	SDI_IOCTL_RX_IO_TIMEOUT,
			/* arg		*/	10,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	SDI_IOCTL_DATA_FORMAT,
			/* arg		*/	SDI_DATA_FORMAT_OFF_BIN,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	SDI_IOCTL_AIN_MODE,
			/* arg		*/	ain_mode,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	SDI_IOCTL_AIN_RANGE,
			/* arg		*/	ain_range,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	SDI_IOCTL_AUTO_CALIBRATE,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	SDI_IOCTL_AIN_BUF_CLEAR,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_SLEEP,
			/* cmd		*/	0,
			/* arg		*/	1,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	static const service_data_t	list_2[]	=
	{
		{
			/* service	*/	SERVICE_IOCTL_SET,
			/* cmd		*/	SDI_IOCTL_INITIALIZE,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST },
	};

	char	buf[64];
	int		bytes;
	int		chan;
	s32		d_adc;
	int		errs;
	int		i;
	int		ret;
	float	v_adc;
	float	v_avg;
	int		v_count[16];	// 16 channels max
	float	v_delta;
	float	v_dif;
	float	v_lsb;
	float	v_sum[16];		// 16 channels max
	float	v_targ;

	gsc_label(title);
	errs	= service_ioctl_set_reg_list(fd, list_1);

	if (errs == 0)
	{
		bytes	= (int) sizeof(_data);
		ret		= sdi_read(fd, _data, bytes);
		errs	= service_ioctl_reg_get_list(fd, list_2);

		if (errs)
		{
		}
		else if (ret < 0)
		{
			errs	= 1;
			printf("FAIL <---  (sdi_read(), error %d)\n", ret);
		}
		else if (ret != bytes)
		{
			errs	= 1;
			printf(	"FAIL <---  (requested %d bytes, got lony %d)\n",
					bytes,
					ret);
		}
	}

	if (errs == 0)
		printf("\n");

	// Calculate the numbers we'll use for testing.
	v_lsb	= range / 32768;
	v_targ	= range * frac;
	v_dif	= acc + v_lsb;

	memset(v_count, 0, sizeof(v_count));
	memset(v_sum, 0, sizeof(v_sum));

	if (errs == 0)
	{
		// Go through the data and sum the content for averaging.

		for (i = 0; i < SIZEOF_ARRAY(_data); i++)
		{
			chan		= (_data[i] & 0xF0000) >> 16;
			d_adc		= _data[i] & 0xFFFF;
			v_adc		= range / 32768 * (d_adc - 0x8000);
			v_sum[chan]	+= v_adc;
			v_count[chan]++;
		}

		gsc_label_level_inc();

		for (i = 0; i < 16; i++)
		{
			if (v_count[i] == 0)
				continue;

			sprintf(buf, "Channel %d", i);
			gsc_label(buf);
			v_avg	= v_sum[i] / v_count[i];
			v_delta	= v_targ - v_avg;

			if (v_delta < -v_dif)
			{
				errs	= 1;
				printf(	"FAIL <---  (low average: got %+.2f mv, min is %+.2f mv)\n",
						v_delta * 1000,
						-v_dif * 1000);
			}
			else if (v_delta > v_dif)
			{
				errs	= 1;
				printf(	"FAIL <---  (high average: got %+.2f mv, max is %+.2f mv)\n",
						v_delta * 1000,
						v_dif * 1000);
			}
			else
			{
				printf("PASS  (ave difference %+.2f mv)\n", v_delta * 1000);
			}
		}

		gsc_label_level_dec();
	}

	return(errs);
}



/******************************************************************************
*
*	Function:	voltage_test
*
*	Purpose:
*
*		Perform ADC voltage accuracy tests.
*
*	Arguments:
*
*		fd		The handle for the device to access.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int voltage_test(int fd)
{
	int	errs	= 0;

	gsc_label("ADC Voltage Tests");
	printf("\n");
	gsc_label_level_inc();

	errs	+= _v_test(fd, "Vref Test @ +-10v",		SDI_AIN_MODE_VREF, SDI_AIN_RANGE_10V,	10.0,	0.0052,	0.99);
	errs	+= _v_test(fd, "Vref Test @ +-5v",		SDI_AIN_MODE_VREF, SDI_AIN_RANGE_5V,	 5.0,	0.0031,	0.99);
	errs	+= _v_test(fd, "Vref Test @ +-2.5v",	SDI_AIN_MODE_VREF, SDI_AIN_RANGE_2_5V,	 2.5,	0.0022,	0.99);
	errs	+= _v_test(fd, "Vref Test @ +-1.25v",	SDI_AIN_MODE_VREF, SDI_AIN_RANGE_1_25V,	 1.25,	0.0015,	0.99);

	errs	+= _v_test(fd, "Zero Test @ +-10v",		SDI_AIN_MODE_ZERO, SDI_AIN_RANGE_10V,	10.0,	0.0012,	0.00);
	errs	+= _v_test(fd, "Zero Test @ +-5v",		SDI_AIN_MODE_ZERO, SDI_AIN_RANGE_5V,	 5.0,	0.0011,	0.00);
	errs	+= _v_test(fd, "Zero Test @ +-2.5v",	SDI_AIN_MODE_ZERO, SDI_AIN_RANGE_2_5V,	 2.5,	0.0009,	0.00);
	errs	+= _v_test(fd, "Zero Test @ +-1.25v",	SDI_AIN_MODE_ZERO, SDI_AIN_RANGE_1_25V,	 1.25,	0.0008,	0.00);

	gsc_label_level_dec();
	return(errs);
}


