// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16SDI/16SDI_Linux_4.x.x.x_GSC_DN/utils/util_ain_mode.c $
// $Rev: 51943 $
// $Date: 2022-10-25 08:29:04 -0500 (Tue, 25 Oct 2022) $

// 16SDI: Utilities: source file

#include "main.h"



/******************************************************************************
*
*	Function:	sdi_ain_mode
*
*	Purpose:
*
*		Provide a visual wrapper for the SDI_IOCTL_AIN_MODE service.
*
*	Arguments:
*
*		fd		Use this handle to access the device.
*
*		index	The index of the device to access. Ignore if < 0.
*				This is for display purposes only.
*
*		verbose	Work verbosely?
*
*		set		The setting to apply.
*
*		get		The current setting is recorded here, if not NULL.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int	sdi_ain_mode(int fd, int index, int verbose, s32 set, s32* get)
{
	char	buf[128];
	int		errs;
	int		ret;

	if (verbose)
		gsc_label_index("Analog Input Mode", index);

	ret		= sdi_ioctl(fd, SDI_IOCTL_AIN_MODE, &set);
	errs	= ret ? 1 : 0;

	switch (set)
	{
		default:

			sprintf(buf, "INVALID: %ld", (long) set);
			break;

		case SDI_AIN_MODE_DIFF:

			strcpy(buf, "Differential");
			break;

		case SDI_AIN_MODE_SE:

			strcpy(buf, "Single Ended");
			break;

		case SDI_AIN_MODE_ZERO:

			strcpy(buf, "Zero Test");
			break;

		case SDI_AIN_MODE_VREF:

			strcpy(buf, "Vref Test");
			break;
	}

	if (verbose)
		printf("%s  (%s)\n", errs ? "FAIL <---" : "PASS", buf);

	if (get)
		get[0]	= set;

	return(errs);
}


