// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16SDI/16SDI_Linux_4.x.x.x_GSC_DN/utils/util_config_ai.c $
// $Rev: 51943 $
// $Date: 2022-10-25 08:29:04 -0500 (Tue, 25 Oct 2022) $

// 16SDI: Utilities: source file

#include "main.h"



/******************************************************************************
*
*	Function:	sdi_config_ai
*
*	Purpose:
*
*		Configure the given device using common defaults.
*
*	Arguments:
*
*		fd		The handle to use to access the driver.
*
*		index	The index of the device to access. Ignore if < 0.
*
*		verbose	Work verbosely?
*
*	Returned:
*
*		>= 0	The number of errors encountered here.
*
******************************************************************************/

int	sdi_config_ai(int fd, int index, int verbose, s32 fsamp)
{
	int	errs	= 0;
	s32	ndiv;
	s32	nrate;

	if (verbose)
	{
		gsc_label_index("Input Configuration", index);
		gsc_label_level_inc();
		printf("\n");
	}

	errs	+= sdi_initialize			(fd, index, verbose);
	errs	+= sdi_rx_io_mode			(fd, index, verbose, GSC_IO_MODE_BMDMA,			NULL);
	errs	+= sdi_rx_io_timeout		(fd, index, verbose, 10,						NULL);
	errs	+= sdi_ain_mode				(fd, index, verbose, SDI_AIN_MODE_DIFF,			NULL);
	errs	+= sdi_ain_range			(fd, index, verbose, SDI_AIN_RANGE_10V,			NULL);
	errs	+= sdi_ain_buf_input		(fd, index, verbose, SDI_AIN_BUF_INPUT_ENABLE,	NULL);
	errs	+= sdi_ain_buf_thresh		(fd, index, verbose, 50000,						NULL);
	errs	+= sdi_sw_sync_mode			(fd, index, verbose, SDI_SW_SYNC_MODE_CLR_BUF,	NULL);
	errs	+= sdi_ch_grp_src_all		(fd, index, verbose, SDI_CH_GRP_SRC_GEN_A);
	errs	+= sdi_channel_order		(fd, index, verbose, SDI_CHANNEL_ORDER_SYNC,	NULL);
	errs	+= sdi_data_format			(fd, index, verbose, SDI_DATA_FORMAT_OFF_BIN,	NULL);
	errs	+= sdi_init_mode			(fd, index, verbose, SDI_INIT_MODE_INITIATOR,	NULL);
	errs	+= sdi_fsamp_ai_compute		(fd, index, verbose, fsamp, &nrate, &ndiv,		NULL);
	errs	+= sdi_rate_div_ndiv_all	(fd, index, verbose, ndiv);
	errs	+= sdi_rate_gen_nrate_all	(fd, index, verbose, nrate);
	errs	+= sdi_fsamp_ai_report_all	(fd, index, verbose, NULL);
	errs	+= sdi_auto_calibrate		(fd, index, verbose);	// Cable may need disconnecting for this.
	errs	+= sdi_ain_buf_clear		(fd, index, verbose);

	if (verbose)
		gsc_label_level_dec();

	return(errs);
}


