// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16SDI/16SDI_Linux_4.x.x.x_GSC_DN/utils/util_irq_sel.c $
// $Rev: 42841 $
// $Date: 2018-05-17 18:06:37 -0500 (Thu, 17 May 2018) $

// 16SDI: Utilities: source file

#include "main.h"



/******************************************************************************
*
*	Function:	sdi_irq_sel
*
*	Purpose:
*
*		Provide a visual wrapper for the SDI_IOCTL_IRQ_SEL service.
*
*	Arguments:
*
*		fd		Use this handle to access the device.
*
*		index	The index of the device to access. Ignore if < 0.
*				This is for display purposes only.
*
*		verbose	Work verbosely?
*
*		set		The setting to apply.
*
*		get		The current setting is recorded here, if not NULL.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int	sdi_irq_sel(int fd, int index, int verbose, s32 set, s32* get)
{
	char	buf[128];
	int		errs;
	int		ret;

	if (verbose)
		gsc_label_index("IRQ Select", index);

	ret		= sdi_ioctl(fd, SDI_IOCTL_IRQ_SEL, &set);
	errs	= ret ? 1 : 0;

	switch (set)
	{
		default:

			sprintf(buf, "INVALID: %ld", (long) set);
			break;

		case SDI_IRQ_INIT_DONE:

			strcpy(buf, "Initialize Done");
			break;

		case SDI_IRQ_AUTO_CAL_DONE:

			strcpy(buf, "Auto-Calibrate Done");
			break;

		case SDI_IRQ_CHANNELS_READY:

			strcpy(buf, "Channels Ready");
			break;

		case SDI_IRQ_BUF_THRESH_L2H:

			strcpy(buf, "Buffer Threshold Low to High");
			break;

		case SDI_IRQ_BUF_THRESH_H2L:

			strcpy(buf, "Buffer Threshold High to Low");
			break;

		case SDI_IRQ_BUF_AE:

			strcpy(buf, "Buffer Almost Empty");
			break;

		case SDI_IRQ_BUF_AF:

			strcpy(buf, "Buffer Almost Full");
			break;
	}

	if (verbose)
		printf("%s  (%s)\n", errs ? "FAIL <---" : "PASS", buf);

	if (get)
		get[0]	= set;

	return(errs);
}


